/*
 * MyPanel.java
 *
 * Created on Utorok, 2004, oktber 12, 20:41
 */

/**
 *
 * @author  m
 */
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.Applet;
import java.util.*;
import javax.swing.*;

public class MyPanel extends JPanel implements Runnable, ImageProducer
{
	int _width;
	int _height;
	Image _image;
	Thread _thread;
	ImageConsumer _consumer;
	DirectColorModel _model;
	int frame = 0;
	float fps = 100.f;
	
	/** Creates a new instance of MyPanel */
	public MyPanel()
	{
	//	this.setBackground(new Color(255,0,0));
	//	this.setDoubleBuffered(false);
		_width = getWidth();
		_height = getHeight();
		_image = Toolkit.getDefaultToolkit().createImage(this);
		_model = new DirectColorModel(32,0x00FF0000,0x000FF00,0x000000FF,0);
	}
	public int resizePanel(int width, int height)
	{
		int started = 0;
		if (_thread!=null && _thread.isAlive())
		{
			stop();
			started = 1;
		}
		_width = width;
		_height = height;
		if(_consumer!=null)
			_consumer.setDimensions(_width,_height);
		_image = Toolkit.getDefaultToolkit().createImage(this);
		return started;
	}
	public void setBounds(int x, int y, int width, int height)
	{
		int started = resizePanel(width,height);
		super.setBounds( x, y, width, height);
		if(started==1)start();
	}
	public void setBounds(Rectangle r)
	{
		int started = resizePanel(r.width,r.height);
		super.setBounds(r);
		if(started==1)start();
	}
	public void setSize(Dimension d)
	{
		int started = resizePanel(d.width,d.height);
		super.setSize(d);
		if(started==1)start();
	}
	public void setSize(int width,int height)
	{
		int started = resizePanel(width,height);
		super.setSize(width,height);
		if(started==1)start();		
	}

	public void start()
	{
        if (_thread==null)					// check thread
        {
			_thread = new Thread(this);		// create thread
            _thread.start();				// start thread
        }		
	}
	
	public void stop()
	{
		if (_thread!=null && _thread.isAlive())		// check thread is valid and alive
		{
			_thread.stop();	// stop thread
		}
		_thread = null;			// null thread
	}
	
	public void run()
	{
		Device2D dv = new Device2D(_width,_height);
		dv.clearColorBuffer(0);
		long time = System.currentTimeMillis();
		frame = 0;
		while(true)
		{
			frame++;
			dv.noiseColor();
			
            update(dv.cBuf);
			if(System.currentTimeMillis() - time > 0)
			{
			//	System.out.println(Long.toString(System.currentTimeMillis() - time));
				float minTime = 0.001f*(float)(System.currentTimeMillis() - time);
				fps = 0.1f*(float)frame/minTime + 0.9f*fps;;
				frame = 0;
				time = System.currentTimeMillis();
			}
        }
	}
	
	public synchronized void addConsumer(java.awt.image.ImageConsumer ic)
	{
		// register image consumer
		_consumer = ic;

		// set image dimensions
		_consumer.setDimensions(_width,_height);

		// set image consumer hints for speed
		_consumer.setHints(ImageConsumer.TOPDOWNLEFTRIGHT|ImageConsumer.COMPLETESCANLINES|ImageConsumer.SINGLEPASS|ImageConsumer.SINGLEFRAME);

		// set image color model
		_consumer.setColorModel(_model);
	}
	
	public synchronized boolean isConsumer(java.awt.image.ImageConsumer ic)
	{
		// check if consumer is registered
		return true;
	}
	
	public synchronized void removeConsumer(java.awt.image.ImageConsumer ic)
	{
		// remove image consumer
	}
	
	public void requestTopDownLeftRightResend(java.awt.image.ImageConsumer ic)
	{
		// ignore resend request
	}
	
	public synchronized void startProduction(java.awt.image.ImageConsumer ic)
	{
		// add consumer
		addConsumer(ic);
	}
	
	public synchronized void update(int[] pixels)
	{
		// check consumer
		if (_consumer!=null && pixels.length==_width*_height )
		{
			// copy integer pixel data to image consumer
			_consumer.setPixels(0,0,_width,_height,_model,pixels,0,_width);

			// notify image consumer that the frame is done
			_consumer.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
		}

		// paint
		if(_image!=null)
		{
			Graphics g = getGraphics();
			g.drawImage(_image,0,0,_width,_height,null);
			
			g.setColor(new Color(255,255,255));
			g.fillRect(10,3, 100, 15);
			g.setColor(new Color(0,0,0));
			g.drawString("FPS: "+Float.toString(fps), 15, 15);
		
			g.setColor(new Color(255,255,255));
			g.fillRect(10,18, 100, 15);
			g.setColor(new Color(0,0,0));
			g.drawString(Float.toString(fps*_width*_height*0.000001f)+" MPx", 15, 30);
		}
	}
	public synchronized void paint(Graphics g)
	{
		if(_image!=null)
		{
			g.drawImage(_image,0,0,_width,_height,null);
			
			g.setColor(new Color(255,255,255));
			g.fillRect(10,3, 70, 15);
			g.setColor(new Color(0,0,0));
			g.drawString(Float.toString(fps), 15, 15);
		}		
	}
}
